@@ -15,6 +15,9 @@
  */
 package net.morilib.awk.builtin;
 
+import net.morilib.awk.misc.AwkComplexMath;
+import net.morilib.awk.value.AwkValue;
+
 /**
  * 組み込み関数cosの記述です。
  *
@@ -27,9 +30,8 @@ public class AwkCos extends AwkMathFunction {
 		super("cos");
 	}
 
-	@Override
-	protected double f(double x) {
-		return Math.cos(x);
+	protected AwkValue f(AwkValue x) {
+		return AwkComplexMath.cos(x);
 	}
 
 }
