@@ -25,6 +25,7 @@ import net.morilib.awk.expr.AwkExpression;
 import net.morilib.awk.misc.System2;
 import net.morilib.awk.namespace.AwkNamespace;
 import net.morilib.awk.value.AwkInteger;
+import net.morilib.awk.value.AwkString;
 import net.morilib.awk.value.AwkValue;
 
 /**
@@ -118,6 +119,8 @@ public class AwkPrint extends AwkExpression {
 			}
 			return AwkValue.TRUE;
 		} catch (IOException e1) {
+			ns.getRoot().assign("ERRNO",
+					AwkString.valueOf(e1.getMessage()));
 			return AwkValue.FALSE;
 		} finally {
 			if(w != null)  w.flush();
