@@ -29,11 +29,16 @@ import java.io.OutputStream;
 public final class AwkVirtualFile {
 
 	static final String STDIN  = "/dev/stdin";
+	static final String STDIN2 = "/dev/fd/0";
+
 	static final String DIALOG = "/dev/dialog";
 
 	static final String STDOUT = "/dev/stdout";
 	static final String STDERR = "/dev/stderr";
+	static final String STDOT2 = "/dev/fd/1";
+	static final String STDER2 = "/dev/fd/2";
 	static final String NULL   = "/dev/null";
+
 	static final String MSGBOX = "/dev/msgbox";
 
 	/**
@@ -73,9 +78,9 @@ public final class AwkVirtualFile {
 	 * @param fs ファイル管理クラス
 	 */
 	public static LineWriter out(String fname, AwkFiles fs) {
-		if(fname.equals(STDOUT)) {
+		if(fname.equals(STDOUT) || fname.equals(STDOT2)) {
 			return fs.getStdout();
-		} else if(fname.equals(STDERR)) {
+		} else if(fname.equals(STDERR) || fname.equals(STDER2)) {
 			return fs.getStderr();
 		} else if(fname.equals(NULL)) {
 			return NULLWRITER;
@@ -91,7 +96,7 @@ public final class AwkVirtualFile {
 	 * @param fs ファイル管理クラス
 	 */
 	public static RecordReader in(String fname, AwkFiles fs) {
-		if(fname.equals(STDIN)) {
+		if(fname.equals(STDIN) || fname.equals(STDIN2)) {
 			return fs.getStdin();
 		} else {
 			return null;
