@@ -27,6 +27,11 @@ import net.morilib.awk.misc.PromptReader;
 import net.morilib.awk.misc.PushbackLineNumberReadable;
 import net.morilib.awk.misc.PushbackLineNumberReader;
 
+/**
+ * awkiumの字句解析器(lexer)です。
+ * 
+ * @author MORIGUCHI, Yuichiro 2013/03/09
+ */
 public final class AwkLexer {
 
 	private static class EndException extends RuntimeException {}
@@ -39,6 +44,8 @@ public final class AwkLexer {
 		FLOAT1, FLOAT2, FLOAT_E1, FLOAT_E2, FLOAT_E3,
 	}
 
+	private static enum St2 { STRING, STRING_ESC }
+
 	private static final Map<Integer, AwkToken> OP1;
 	private static final Map<String, AwkToken> RES1;
 
@@ -90,8 +97,9 @@ public final class AwkLexer {
 	private int blocks = 0;
 
 	/**
+	 * lexerを生成します。
 	 * 
-	 * @param rd
+	 * @param rd ソースの入力Reader
 	 */
 	public AwkLexer(Reader rd) throws IOException {
 		reader  = new PushbackLineNumberReader(rd);
@@ -101,8 +109,12 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * lexerを生成します。
 	 * 
-	 * @param rd
+	 * @param prompt1 1行目のプロンプト
+	 * @param prompt2 2行目以降のプロンプト
+	 * @param rd ソースの入力Reader
+	 * @throws IOException
 	 */
 	public AwkLexer(String prompt1, String prompt2,
 			Reader rd) throws IOException {
@@ -113,8 +125,9 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * 現在トークンを得ます。
 	 * 
-	 * @return
+	 * @return トークン
 	 */
 	public AwkToken getToken() throws IOException {
 		if(reset) {
@@ -127,8 +140,9 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * トークンを1つ読み、そのトークンを返します。
 	 * 
-	 * @return
+	 * @return トークン
 	 * @throws IOException
 	 */
 	public AwkToken nextToken() throws IOException {
@@ -145,8 +159,10 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * トークンを1つ読み、そのトークンを返し、
+	 * ブロック階層を1つ増やします。
 	 * 
-	 * @return
+	 * @return トークン
 	 * @throws IOException
 	 */
 	public AwkToken nextTokenBegin() throws IOException {
@@ -160,6 +176,7 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * トークンが行の終わりにあるときtrueを得ます。
 	 * 
 	 * @return
 	 * @throws IOException
@@ -176,6 +193,7 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * 行の終わりのトークンを廃棄します。
 	 * 
 	 * @return
 	 * @throws IOException
@@ -194,10 +212,13 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * トークンが与えられた引数と一致するときそのトークンを廃棄し、
+	 * 違うときは例外を返します。
 	 * 
-	 * @param t
-	 * @return
+	 * @param t あるべきトークン
+	 * @return 次のトークン
 	 * @throws IOException
+	 * @throws AwkSyntaxException トークンが一致しないとき
 	 */
 	public AwkToken eatToken(AwkToken t) throws IOException {
 		if(!token.equals(t)) {
@@ -207,10 +228,13 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * トークンが与えられた引数と一致し、さらに行の最後のとき
+	 * そのトークンを廃棄し、違うときは例外を返します。
 	 * 
-	 * @param t
-	 * @return
+	 * @param t あるべきトークン
+	 * @return 次のトークン
 	 * @throws IOException
+	 * @throws AwkSyntaxException トークンが一致しないとき
 	 */
 	public AwkToken eatTokenEnd(AwkToken t) throws IOException {
 		blocks--;
@@ -225,9 +249,11 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * トークンが与えられた引数と一致するときそのトークンを廃棄し、
+	 * 違うときは何もしません。
 	 * 
-	 * @param t
-	 * @return
+	 * @param t あるべきトークン
+	 * @return 次のトークン
 	 * @throws IOException
 	 */
 	public AwkToken eatTokenOpt(AwkToken t) throws IOException {
@@ -236,8 +262,9 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * 正規表現パターンを取得します。
 	 * 
-	 * @return
+	 * @return 正規表現パターン
 	 * @throws IOException
 	 */
 	public String getPattern() throws IOException {
@@ -245,15 +272,16 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * 行番号を取得します。
 	 * 
-	 * @return
+	 * @return 行番号
 	 */
 	public int getLineNumber() {
 		return reader.getLineNumber();
 	}
 
 	/**
-	 * 
+	 * プロンプトをリセットし、1行目のものにします。
 	 */
 	public void resetPrompt() throws IOException {
 		reader.resetPrompt();
@@ -261,6 +289,7 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * 入力ソースストリームが改行のときtrueを得ます。
 	 * 
 	 * @return
 	 */
@@ -359,11 +388,16 @@ public final class AwkLexer {
 				gethex1(rd) * 0x10   + gethex1(rd));
 	}
 
+	private static int gethex2(
+			PushbackLineNumberReadable rd) throws IOException {
+		return gethex1(rd) * 0x10   + gethex1(rd);
+	}
+
 	static AwkToken _getToken(
 			PushbackLineNumberReadable rd) throws IOException {
 		StringBuffer b1 = new StringBuffer();
 		St1 stat = St1.INIT;
-		int c;
+		int c = 0;
 
 		while(true) {
 			switch(stat) {
@@ -564,13 +598,23 @@ public final class AwkLexer {
 					b1.append('\n');
 				} else if(c == 'r') {
 					b1.append('\r');
+				} else if(c == 'a') {
+					b1.append('\u0007');
 				} else if(c == 'b') {
 					b1.append('\b');
+				} else if(c == 'f') {
+					b1.append('\u000c');
+				} else if(c == 't') {
+					b1.append('\u0009');
+				} else if(c == 'v') {
+					b1.append('\u000b');
 				} else if(isnum(c)) {
 					rd.unread(c);
 					b1.append((char)getoct(rd));
 				} else if(c == 'u') {
 					b1.append((char)gethex(rd));
+				} else if(c == 'x') {
+					b1.append((char)gethex2(rd));
 				} else {
 					b1.appendCodePoint(c);
 				}
@@ -696,9 +740,10 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * トークンを取得します。
 	 * 
-	 * @param rd
-	 * @return
+	 * @param rd 入力ソースのReader
+	 * @return トークン
 	 * @throws IOException
 	 */
 	public static AwkToken getToken(
@@ -711,9 +756,10 @@ public final class AwkLexer {
 	}
 
 	/**
+	 * パターンを取得します。
 	 * 
-	 * @param rd
-	 * @return
+	 * @param rd 入力ソースのReader
+	 * @return パターン
 	 * @throws IOException
 	 */
 	public static String getPattern(
@@ -738,4 +784,107 @@ public final class AwkLexer {
 		}
 	}
 
+	//
+	private static int getoct(String s, int i) {
+		try {
+			if(i + 2 >= s.length())  return -1;
+			return Integer.parseInt(s.substring(i, i + 3), 8);
+		} catch(NumberFormatException e) {
+			return '?';
+		}
+	}
+
+	private static int gethex(String s, int i) {
+		try {
+			if(i + 3 >= s.length())  return -1;
+			return Integer.parseInt(s.substring(i, i + 4), 16);
+		} catch(NumberFormatException e) {
+			return '?';
+		}
+	}
+
+	private static int gethex2(String s, int i) {
+		try {
+			if(i + 1 >= s.length())  return -1;
+			return Integer.parseInt(s.substring(i, i + 2), 16);
+		} catch(NumberFormatException e) {
+			return '?';
+		}
+	}
+
+	/**
+	 * 
+	 * @param s
+	 * @return
+	 */
+	public static String escape(String s) {
+		StringBuffer b1 = new StringBuffer();
+		St2 stat = St2.STRING;
+		int c, d;
+
+		for(int i = 0; i < s.length(); i++) {
+			c = s.charAt(i);
+			switch(stat) {
+			case STRING:
+				if(c == '\\') {
+					stat = St2.STRING_ESC;
+				} else {
+					b1.appendCodePoint(c);
+				}
+				break;
+			case STRING_ESC:
+				if(c == '\"') {
+					b1.append('\"');
+				} else if(c == 'n') {
+					b1.append('\n');
+				} else if(c == 'r') {
+					b1.append('\r');
+				} else if(c == 'a') {
+					b1.append('\u0007');
+				} else if(c == 'b') {
+					b1.append('\b');
+				} else if(c == 'f') {
+					b1.append('\u000c');
+				} else if(c == 't') {
+					b1.append('\u0009');
+				} else if(c == 'v') {
+					b1.append('\u000b');
+				} else if(isnum(c)) {
+					if((d = getoct(s, i)) < 0) {
+						b1.append('\\');
+						b1.append(s.substring(i));
+						return b1.toString();
+					} else {
+						b1.append((char)d);
+						i += 2;
+					}
+				} else if(c == 'u') {
+					if((d = gethex(s, i + 1)) < 0) {
+						b1.append('\\');
+						b1.append(s.substring(i));
+						return b1.toString();
+					} else {
+						b1.append((char)d);
+						i += 4;
+					}
+				} else if(c == 'x') {
+					if((d = gethex2(s, i + 1)) < 0) {
+						b1.append('\\');
+						b1.append(s.substring(i));
+						return b1.toString();
+					} else {
+						b1.append((char)d);
+						i += 2;
+					}
+				} else {
+					b1.appendCodePoint(c);
+				}
+				stat = St2.STRING;
+				break;
+			}
+		}
+		if(stat.equals(St2.STRING_ESC))  b1.append('\\');
+		return b1.toString();
+	}
+
 }
